#!/usr/local/bin/perl

##########
#
# Common functions used by some scripts
#
#
##########

use CGI::Carp qw(fatalsToBrowser);
use configwm;
use URI::Escape qw(uri_escape uri_unescape);
use MIME::Parser;

package common;

sub new {
	my $self={};
	($self->{modname})=@_;
	bless $self;
	return $self;
}

# function to encrypt the password
sub cryptit {
	my ($self,$string,$pkey)=@_;
	my $crypted="";
	my $i;
	my $fullkey=$conf::keycrypt.$conf::keycrypt;
	$fullkey=$conf::keycrypt.$pkey if (defined($conf::cookiedomain));
	for ($i=0;$i<length($string);$i++) {
		my $tmpcrypt=ord(substr($string,$i,1))+ord(substr($fullkey,(abs(cos($i))*6),1));
		my $crypt1=chr(int($tmpcrypt/2));
		my $crypt2=chr($tmpcrypt-int($tmpcrypt/2));
		$crypted.=$crypt1.$crypt2;
	}
	# escape everything not a letter (smssms at attglobal.net)
	$string= URI::Escape::uri_escape($crypted,"^A-Za-z");
	undef($crypt);
	return $string;
}

# function to decrypt the password
sub decryptit {
	my ($self,$string,$pkey)=@_;
	my $decrypted="";
	my $i;
	my $fullkey=$conf::keycrypt.$conf::keycrypt;
	$fullkey=$conf::keycrypt.$pkey if (defined($conf::cookiedomain));
	$string=URI::Escape::uri_unescape($string);
	for ($i=0;$i<length($string);$i=$i+2) {
		my $decrypt1=substr($string,$i,1);
		my $decrypt2=substr($string,$i+1,1);
		my $tmpdecrypt=ord($decrypt1)+ord($decrypt2); 
		$decrypted.=chr($tmpdecrypt-ord(substr($fullkey,(abs(cos(int($i/2)))*6),1)));
	}
	$string=$decrypted;
	undef($decrypted);
	return $string;
}

# function to decode MIME-encoded mails
sub decodemail {
	my ($self,$entity,$attach,$preformat)=@_;
	my $messagestored="";
	my $bodysend="";
	my $i;
	my $j;
	my $k=0;
	if ($entity->parts) {
		for ($i=0;$i<$entity->parts;$i++) {   
			my $ent=$entity->parts($i);  
      
			my $type = lc($ent->mime_type);
			# Get the body, as a MIME::Body;
			my $body = $ent->bodyhandle;
			if ($body) {
				# Where's the data?
				# text data
				if ($type=~ m/text/) {
					if ($type=~ m/text\/plain/) {
						$messagestored.="<hr>" if (($i > 0)&&($preformat));
						if ($preformat) {
							$messagestored.=htmlize($body->as_string);
						} else {
							$messagestored.= $body->as_string;
						}
					} elsif ($type=~ m/text\/html/) {
						$messagestored.= $body->as_string;
					}
					$ent->purge;
				# images
				} elsif ($type=~ m/image\//) {
					$messagestored.="<hr>" if (($i > 0)&&($preformat));
					$messagestored.="<br><img src=\"" if ($preformat);
					my $path = $body->path;
					$path=~ s/$conf::temppath\/?(.*)/$conf::tempurl\/$1/;
					my $fichn=$1;
					$messagestored.="\n[ $fichn ]\n" if (!$preformat);
					$messagestored.="$path\">" if ($preformat);
					$attach->[$k]=$path;
					$k++;
				# message forwarded as message/RFC
				} elsif ($type=~ m/message\/rfc/) {
					my $attachinside=[];
					my $parser = new MIME::Parser;
					$parser->output_dir($conf::temppath);
					$parser->output_prefix("msginside");
					my $entityinside= $parser->parse_data($body->as_string);
					$messagestored.=decodemail($self,$entityinside,$attachinside,$preformat);
					my $l;
					for ($l=0; $l< scalar(@{$attachinside}); $l++) { $attach->[$k]=$attachinside->[$l]; $k++; }
				} else {		# data is on disk:
					my $path = $body->path;
					$path=~ s/$conf::temppath\/?(.*)/$conf::tempurl\/$1/;
					$attach->[$k]=$path;
					$k++;
				}    
			# we may have multipart included inside multipart
			} elsif  (($type =~ m/multipart/)||($type=~ m/message\/rfc/)) {
				my $attachinside=[];
				my $entityinside= $ent;
				$messagestored.=decodemail($self,$entityinside,$attachinside,$preformat);
				my $l;
				for ($l=0; $l< scalar(@{$attachinside}); $l++) { $attach->[$k]=$attachinside->[$l]; $k++; }
			} 
		}	
	} else {
		$body = $entity->bodyhandle;
		$type = lc($entity->mime_type);
		if ($type=~ m/text/ || $type=~ m/message/) {
			$messagestored.="<hr>" if ($preformat);
			if ($preformat && $type !~ /html/) {
				$messagestored.= htmlize($body->as_string);
			} else {
				$messagestored.= $body->as_string;
			}
		}
		$entity->purge;	
	}
	return $messagestored;
} 

# function to "beautify" HTML code
sub htmlize {
	my $lineall;
	my $out="<pre>\n";
	my $line;
	foreach $lineall (@_) {
	    foreach $line (split("\n",$lineall)) {
		# split lines longer than 80 chars (don't split quoted!)
		$line2="";
		while (length($line) > 80 && $line!~/^[>|:]\s/) {
			if ( $line=~s;^(.{65,78})\s(.+)$;$2; ) {
				$line2.="$1\n";
			} else {
				# can't split on char 68-78!
				last;
			}
		}
		$line=$line2.$line;

	        # char < and > (spaces before/after lt/gt are important!)
		$line=~s,<, &lt; ,g;
		$line=~s,>, &gt; ,g;
		# order of following lines is dependent!
		# quoting (in color!)
		$line=~s,^ &gt; (.*),<font color="#666666"><i> &gt; $1 </i></font>,;
		$line=~s,^(:\s+.*),<font color="#666666"><i>$1</i></font>,;
		# e-mail
		$line=~s,(\w\S+@\S+\w)\.*,<a href="mailto:$1">$1</a>,g;
		# http and ftp
		$line=~s,(h*[t|f]tp://[^\)|\s|\,]+\w[^\.\s\)])([\)\.]*),<a href="$1">$1</a>$2,g;
		# nuke spaces around &lt; | &gt;
		$line=~s, (&[l|g]t;) ,$1,g;
		$out.="$line\n";
	    }
	}
	$out.="</pre>";
	return $out;
}


alarm($conf::cgi_timeout);
$SIG{ALRM} = \&myalrm;
sub myalrm {
        die "Host is overloaded. Try again later...";
}

# the menu bar for non-frames version
sub startmenu {
	my ($self,$loginname,$POPserver,$cache,$cpassword)=@_;
	
    print "<table border=0 cellspacing=0 cellpadding=4>\n";
    print "<tr><td bgcolor=$conf::lightbgcolormenu valign=top width=100><font size=-2 face=\"arial\" color=$conf::linkcolormenu><br>";
    print "<a href=\"sendform.pl?loginname=$loginname\&POPserver=$POPserver\&cache=$cache\&password=$cpassword\"><IMG SRC=\"$conf::PATH_NSWM/newmail.gif\" border=0 align=left>$conf::newmsgtext</a><br><br>\n";
    print "<a href=\"inbox.pl?loginname=$loginname\&POPserver=$POPserver\&cache=$cache\&password=$cpassword\"><IMG SRC=\"$conf::PATH_NSWM/inbox.gif\" border=0 align=left>$conf::inboxtext</a><br><br>\n";
    print "<a href=\"sentmail.pl?loginname=$loginname\&POPserver=$POPserver\&cache=$cache\&password=$cpassword\"><IMG SRC=\"$conf::PATH_NSWM/sentmail.gif\" border=0 align=left>$conf::sentmsgtext</a><br><br>\n";
    print "<a href=\"login.pl\"><IMG SRC=\"$conf::PATH_NSWM/exit.gif\" BORDER=0 align=left>$conf::logouttext</a><BR>\n";
    print "</font></td><td>";
								
}

# finishing the page when using non-frame version
sub stopmenu {
	my ($self)=@_;
	print "</td></tr></table>";
}

return 1;
